@@ -3,8 +3,10 @@
 # Copyright (C) 2012-2013 The Python Software Foundation.
 # See LICENSE.txt and CONTRIBUTORS.txt.
 #
+from __future__ import unicode_literals
 
 import base64
+import io
 import os
 import hashlib
 import logging
@@ -582,6 +584,19 @@ class TestDatabase(LoggingCatcher,
         import cgi
         self.assertIs(e.value, cgi.print_directory)
 
+        # See issue #78. Test reading an entry_points.txt with leading spaces
+
+        TEST_EXPORTS = b"""
+        [paste.server_runner]
+        main = waitress:serve_paste
+        [console_scripts]
+        waitress-serve = waitress.runner:run
+        """
+        with io.BytesIO(TEST_EXPORTS) as f:
+            exports = read_exports(f)
+        self.assertEqual(set(exports.keys()),
+                         set(['paste.server_runner', 'console_scripts']))
+
     def test_exports_iteration(self):
         d = DistributionPath()
         expected = set((
@@ -687,7 +702,7 @@ class DataFilesTestCase(GlobTestCaseBase):
                 'you/kill/pandabear.guy': None}
         self.assertRulesMatch(rules, spec)
 
-    def test_final_exemple_glob(self):
+    def test_final_example_glob(self):
         rules = [
             ('mailman/database/schemas/', '*', '{appdata}/schemas'),
             ('', os.path.join('**', '*.tpl'), '{appdata}/templates'),
@@ -855,6 +870,7 @@ class DepGraphTestCase(LoggingCatcher,
         self.checkLists(graph.missing.get(cheese, []), [])
 
     def test_dependent_dists(self):
+        # import pdb; pdb.set_trace()
         dists = self.get_dists(self.DISTROS_DIST)
 
         choxie, grammar, towel = dists
