@@ -18,11 +18,12 @@ import tempfile
 from compat import unittest
 
 from distlib import DistlibException
-from distlib.compat import ZipFile, sysconfig
+from distlib.compat import ZipFile, sysconfig, fsencode
 from distlib.database import DistributionPath, InstalledDistribution
 from distlib.manifest import Manifest
 from distlib.metadata import Metadata, METADATA_FILENAME
 from distlib.scripts import ScriptMaker
+from distlib.util import get_executable
 from distlib.wheel import (Wheel, PYVER, IMPVER, ARCH, ABI, COMPATIBLE_TAGS,
                            is_compatible)
 
@@ -107,6 +108,13 @@ class WheelTestCase(unittest.TestCase):
               ['linux_x86_64'])),
             ('test-1.0-1st-py2.py3-none-win32.whl',
              ('test', '1.0', '1st', ['py2', 'py3'], ['none'], ['win32'])),
+            ('Pillow-2.8.1-cp27-none-macosx_10_6_intel.'
+             'macosx_10_9_intel.macosx_10_9_x86_64.macosx_10_10_intel.'
+             'macosx_10_10_x86_64.whl',
+             ('Pillow', '2.8.1', '', ['cp27'], ['none'],
+              ['macosx_10_6_intel', 'macosx_10_9_intel',
+               'macosx_10_9_x86_64', 'macosx_10_10_intel',
+               'macosx_10_10_x86_64'])),
         )
 
         for name, values in cases:
@@ -314,6 +322,34 @@ class WheelTestCase(unittest.TestCase):
         self.assertFalse(hasattr(warner, 'wheel_version'))
         self.assertFalse(hasattr(warner, 'file_version'))
 
+    def test_custom_executable(self):
+        fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
+        for executable in 'mypython', None:
+            dstdir = tempfile.mkdtemp()
+            self.addCleanup(shutil.rmtree, dstdir)
+            w = Wheel(fn)
+            paths = {'prefix': dstdir}
+            for key in ('purelib', 'platlib', 'headers', 'scripts', 'data'):
+                paths[key] = os.path.join(dstdir, key)
+            maker = ScriptMaker(None, None)
+            maker.variants = set([''])
+            maker.executable = executable
+            w.install(paths, maker)
+            # On Windows there will be an exe file, and on POSIX a text file.
+            # The test is structured to not care.
+            p = paths['scripts']
+            # there should be just one file in the directory - dummy.py/dummy.exe
+            p = os.path.join(p, os.listdir(p)[0])
+            with open(p, 'rb') as f:
+                data = f.read()
+            if executable is None:
+                expected = fsencode(get_executable())
+            else:
+                expected = executable.encode('utf-8')
+            expected = b'#!' + expected + b' -E'
+            if not sysconfig.is_python_build():
+                self.assertIn(expected, data)
+
     def test_verify(self):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
         w = Wheel(fn)
@@ -335,7 +371,7 @@ class WheelTestCase(unittest.TestCase):
     def wheel_modifier_ver(self, path_map):
         mdpath = path_map['dummy-0.1.dist-info/pydist.json']
         md = Metadata(path=mdpath)
-        md.version = '0.1-123'
+        md.version = '0.1+123'
         md.write(path=mdpath)
         return True
 
@@ -358,7 +394,7 @@ class WheelTestCase(unittest.TestCase):
         w.verify()
         md = w.metadata
         self.assertEqual(md.run_requires, [{'requires': ['numpy']}])
-        self.assertEquals(md.version, '0.1-1')
+        self.assertEquals(md.version, '0.1+1')
 
         modified = w.update(self.wheel_modifier_ver)
         self.assertTrue(modified)
@@ -367,7 +403,7 @@ class WheelTestCase(unittest.TestCase):
         w.verify()
         md = w.metadata
         self.assertEqual(md.run_requires, [{'requires': ['numpy']}])
-        self.assertEquals(md.version, '0.1-123')
+        self.assertEquals(md.version, '0.1+123')
 
     def test_info(self):
         fn = os.path.join(HERE, 'dummy-0.1-py27-none-any.whl')
@@ -438,14 +474,18 @@ class WheelTestCase(unittest.TestCase):
             parts.append('m')
         if sysconfig.get_config_var('Py_UNICODE_SIZE') == 4:
             parts.append('u')
-        self.assertEqual(''.join(parts), ABI)
+        if sys.version_info[:2] < (3, 5):
+            abi = ABI
+        else:
+            abi = ABI.split('-')[0]
+        self.assertEqual(''.join(parts), abi)
 
-    @unittest.skipIf('SKIP_SLOW' in os.environ, 'Skipping slow test')
+    @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
     def test_build_and_install_pure(self):
         self.do_build_and_install('sarge == 0.1')
 
-    @unittest.skipIf('SKIP_SLOW' in os.environ, 'Skipping slow test')
+    @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipIf(hasattr(sys, 'pypy_version_info'), 'The test distribution'
                                                ' does not build on PyPy')
     @unittest.skipIf(sys.platform != 'linux2', 'The test distribution only '
@@ -454,14 +494,14 @@ class WheelTestCase(unittest.TestCase):
     def test_build_and_install_plat(self):
         self.do_build_and_install('hiredis == 0.1.1')
 
-    @unittest.skipIf('SKIP_SLOW' in os.environ, 'Skipping slow test')
+    @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipIf(sys.version_info[0] == 3, 'The test distribution is not '
                                                '3.x compatible')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
     def test_build_and_install_data(self):
-        self.do_build_and_install('Werkzeug == 0.4')
+        self.do_build_and_install('Werkzeug == 0.5')
 
-    @unittest.skipIf('SKIP_SLOW' in os.environ, 'Skipping slow test')
+    @unittest.skipIf('SKIP_ONLINE' in os.environ, 'Skipping online test')
     @unittest.skipIf(sys.version_info[0] == 3, 'The test distribution is not '
                                                '3.x compatible')
     @unittest.skipUnless(PIP_AVAILABLE, 'pip is needed for this test')
@@ -469,4 +509,7 @@ class WheelTestCase(unittest.TestCase):
         self.do_build_and_install('Babel == 0.9.6')
 
 if __name__ == '__main__':  # pragma: no cover
+    import logging
+    logging.basicConfig(level=logging.DEBUG, filename='test_wheel.log',
+                        filemode='w', format='%(message)s')
     unittest.main()
